from rest_framework import serializers
from .models import TestModel, RelatedModel


class RelatedModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = RelatedModel
        fields = "__all__"


class TestModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = TestModel
        fields = "__all__"



import unittest
from django.test import TestCase
from django.apps import apps
from django.conf import settings
from django_ts_exporter.models_exporter import ModelsExporter

# Configure settings for the Django test environment if not already configured
if not settings.configured:
    settings.configure(
        INSTALLED_APPS=[
            "django.contrib.contenttypes",
            "django.contrib.auth",
            "tests",
        ],
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": ":memory:",
            }
        },
    )

apps.populate(settings.INSTALLED_APPS)


class ModelsExporterTestCase(TestCase):
    def test_model_export(self):
        exporter = ModelsExporter(
            outdir="./tests/typescript/", apps=["tests"], enable_logs=True
        )
        exporter.export()

        with open("./tests/typescript/tests/TestModel.ts", "r", encoding="utf-8") as f:
            content = f.read()

        expected_content = """import { RelatedModel } from "./RelatedModel.ts";

export interface TestModel {
  id: number;
  char_field: string;
  integer_field: number;
  boolean_field: boolean;
  datetime_field: string;
  date_field: string;
  decimal_field: number;
  uuid_field: string;
  json_field: { [key: string]: any };
  foreign_key: RelatedModel;
  one_to_one: RelatedModel;
  file_field: File;
  many_to_many: RelatedModel[];
}

"""
        self.assertEqual(content.strip(), expected_content.strip())


if __name__ == "__main__":
    unittest.main()



import os
import inspect
import importlib
from django.apps import apps
from rest_framework import serializers
from .utils import create_directory


class SerializersExporter:
    def __init__(self, outdir, apps, enable_logs):
        self.outdir = outdir
        self.apps = apps
        self.enable_logs = enable_logs
        self.any_fields_log = []

    def export(self):
        for app in self.apps:
            app_config = apps.get_app_config(app.split(".")[-1])
            serializers_module = f"{app_config.name}.serializers"

            if importlib.util.find_spec(serializers_module):
                module = importlib.import_module(serializers_module)
                self.export_serializers(module, app_config)

            serializers_path = os.path.join(app_config.path, "serializers")
            if os.path.isdir(serializers_path):
                for root, _, files in os.walk(serializers_path):
                    for file in files:
                        if file.endswith(".py") and file != "__init__.py":
                            module_name = f"{app_config.name}.serializers.{file[:-3]}"
                            module = importlib.import_module(module_name)
                            self.export_serializers(module, app_config)

        if self.enable_logs:
            self.log_any_fields()

    def export_serializers(self, module, app_config):
        app_outdir = os.path.join(self.outdir, app_config.name)
        create_directory(app_outdir)

        for name, obj in inspect.getmembers(module, inspect.isclass):
            if (
                issubclass(obj, serializers.ModelSerializer)
                and obj.__module__ == module.__name__
            ):
                ts_interface, imports = self.convert_serializer_to_ts_interface(
                    name, obj, app_config.name
                )
                self.write_interface(app_outdir, name, ts_interface, imports)

    def get_related_type_and_import(
        self, related_model, related_serializer, app_name, serializer_name
    ):
        if related_serializer != serializer_name:  # Avoid self-import
            if related_model._meta.app_label == app_name:
                import_path = f"./{related_serializer}.ts"
            else:
                import_path = (
                    f"../{related_model._meta.app_label}/{related_serializer}.ts"
                )
            return (
                related_serializer,
                f'import {{ {related_serializer} }} from "{import_path}";',
            )
        else:
            return related_serializer, None

    def infer_serializer_method_field_type(self, field, serializer, field_name):
        method_name = field.method_name or f"get_{field_name}"
        method = getattr(serializer, method_name, None)
        if method is None:
            return "any"  # Unable to determine type if the method is not found

        method_source = inspect.getsource(method)

        signature = inspect.signature(method)
        return_annotation = signature.return_annotation

        if return_annotation is inspect.Signature.empty:
            if "return" in method_source:
                return_statements = [
                    line.strip()
                    for line in method_source.split("\n")
                    if line.strip().startswith("return")
                ]
                if return_statements:
                    return_statement = return_statements[-1]
                    if "{" in return_statement and "}" in return_statement:
                        return "{ [key: string]: any }"  # Infer as dictionary
                    elif "[" in return_statement and "]" in return_statement:
                        return "any[]"  # Infer as list
                    elif '"' in return_statement or "'" in return_statement:
                        return "string"
                    elif return_statement.replace("return", "").strip().isdigit():
                        return "number"
                    elif "True" in return_statement or "False" in return_statement:
                        return "boolean"
                    else:
                        return "any"
            return "any"  # Default to 'any' if no type hint is provided

        if return_annotation is int:
            return "number"
        elif return_annotation is str:
            return "string"
        elif return_annotation is bool:
            return "boolean"
        elif return_annotation is float:
            return "number"
        elif return_annotation is list:
            return (
                "any[]"  # More specific type inference for lists can be added if needed
            )
        elif return_annotation is dict:
            return "{ [key: string]: any }"  # More specific type inference for dicts can be added if needed

        if isinstance(return_annotation, type):
            return return_annotation.__name__

        return "any"

    def get_ts_type(
        self, field, field_name, app_name, serializer_name, original_serializer=None
    ):
        if isinstance(field, serializers.CharField):
            return "string", None
        elif isinstance(field, serializers.IntegerField):
            return "number", None
        elif isinstance(field, serializers.BooleanField):
            return "boolean", None
        elif isinstance(field, serializers.DateTimeField):
            return "string", None  # DateTimeField can be represented as ISO 8601 string
        elif isinstance(field, serializers.DateField):
            return "string", None  # DateField can be represented as ISO 8601 string
        elif isinstance(field, serializers.DecimalField):
            return "number", None  # DecimalField can be represented as number
        elif isinstance(field, serializers.ChoiceField):
            ts_type, enum_definition = self.handle_field_with_choices(
                field, field_name, serializer_name
            )
            return ts_type, enum_definition
        elif isinstance(field, serializers.PrimaryKeyRelatedField):
            related_model = field.queryset.model
            related_serializer = related_model.__name__ + "Serializer"
            return self.get_related_type_and_import(
                related_model, related_serializer, app_name, serializer_name
            )
        elif isinstance(field, serializers.ManyRelatedField):
            related_model = field.child_relation.queryset.model
            related_serializer = related_model.__name__ + "Serializer"
            ts_type, import_statement = self.get_related_type_and_import(
                related_model, related_serializer, app_name, serializer_name
            )
            return f"{ts_type}[]", import_statement
        elif isinstance(field, serializers.FileField):
            return "File", None
        elif isinstance(field, serializers.ListSerializer):
            child_serializer = field.child
            related_serializer = child_serializer.__class__.__name__
            ts_type, import_statement = self.get_related_type_and_import(
                child_serializer.Meta.model,
                related_serializer,
                app_name,
                serializer_name,
            )
            return f"{ts_type}[]", import_statement
        elif isinstance(field, serializers.SerializerMethodField):
            inferred_type = self.infer_serializer_method_field_type(
                field, original_serializer, field_name
            )
            return inferred_type, None
        elif isinstance(field, serializers.UUIDField):
            return "string", None  # UUIDField can be represented as string
        elif isinstance(field, serializers.JSONField):
            return (
                "{ [key: string]: any }",
                None,
            )  # JSONField can be represented as dictionary
        elif issubclass(type(field), serializers.ModelSerializer):
            nested_serializer = field.__class__.__name__
            return self.get_related_type_and_import(
                field.Meta.model, nested_serializer, app_name, serializer_name
            )
        return "any", None

    def handle_field_with_choices(self, field, field_name, serializer_name):
        enum_name = f"{serializer_name}{field_name.capitalize()}Enum"
        choices = field.choices
        enum_definition = f"export enum {enum_name} {{\n"
        for value, label in choices.items():
            enum_definition += (
                f"  {label.replace(' ', '_').replace('�', 'í')} = '{value}',\n"
            )
        enum_definition += "}\n"
        return enum_name, enum_definition

    def convert_serializer_to_ts_interface(self, name, serializer, app_name):
        fields = serializer().get_fields()
        ts_fields = []
        imports = set()
        enums = set()

        for field_name, field in fields.items():
            ts_type, enum_definition = self.get_ts_type(
                field, field_name, app_name, name, original_serializer=serializer
            )
            ts_fields.append(f"{field_name}: {ts_type};")

            if ts_type == "any":
                self.any_fields_log.append(f"{app_name}.{name}.{field_name}")
            if enum_definition:
                if "export enum" in enum_definition:
                    enums.add(enum_definition)
                elif enum_definition:
                    imports.add(enum_definition)

        import_statements = "\n".join(sorted(imports))
        enum_statements = "\n".join(sorted(enums))

        interface = (
            f"export interface {name} {{\n  " + "\n  ".join(ts_fields) + "\n}\n\n"
        )

        result = ""

        if import_statements:
            result = f"{import_statements}\n\n"

        if enum_statements:
            result = result + f"{enum_statements}\n\n"

        if interface:
            result = result + f"{interface}"

        return result, list(imports)

    def write_interface(self, app_outdir, name, ts_interface, imports):
        interface_path = os.path.join(app_outdir, f"{name}.ts")
        with open(interface_path, "w", encoding="utf-8") as f:
            f.write(ts_interface)

    def log_any_fields(self):
        if self.any_fields_log:
            print("\nFields with 'any' type detected:")
            for field in self.any_fields_log:
                print(field)
        else:
            print("\nNo fields with 'any' type detected.")



import os
import sys
import django
import argparse
from pathlib import Path
from django.apps import apps
from django.conf import settings
from .serializers_exporter import SerializersExporter
from .models_exporter import ModelsExporter


class TypeScriptExporter:
    def __init__(self, outdir, exclude, enable_logs, source):
        self.outdir = outdir
        self.exclude = exclude
        self.enable_logs = enable_logs
        self.source = source

    def find_django_settings_module(self):
        current_dir = os.getcwd()
        manage_py_path = os.path.join(current_dir, "manage.py")
        if not os.path.exists(manage_py_path):
            raise FileNotFoundError(
                "manage.py not found. Please run this script from the root of your Django project."
            )

        with open(manage_py_path, "r") as f:
            for line in f:
                if "os.environ.setdefault" in line and "DJANGO_SETTINGS_MODULE" in line:
                    parts = line.split(",")
                    if len(parts) > 1:
                        settings_part = parts[1].strip().strip(")")
                        settings_module = settings_part.strip().strip("'\"")
                        return settings_module

        raise RuntimeError(
            "Could not determine the Django settings module. Please ensure DJANGO_SETTINGS_MODULE is set in manage.py."
        )

    def get_local_apps(self):
        local_apps = []
        installed_apps = settings.INSTALLED_APPS
        base_dir = settings.BASE_DIR

        for app in installed_apps:
            app_path = os.path.join(base_dir, app.replace(".", "/"))
            if os.path.isdir(app_path) and app not in self.exclude:
                local_apps.append(app)
        return local_apps

    def find_project_root(self):
        current_dir = Path(__file__).resolve().parent
        while current_dir != current_dir.root:
            if (current_dir / "manage.py").exists():
                return current_dir
            current_dir = current_dir.parent
        raise RuntimeError(
            "Could not find the project root. Please ensure this script is run within a Django project."
        )

    def run(self):
        settings_module = self.find_django_settings_module()
        project_root = self.find_project_root()
        sys.path.append(str(project_root))
        os.environ.setdefault("DJANGO_SETTINGS_MODULE", settings_module)

        try:
            django.setup()
        except Exception as e:
            print(f"Error during django.setup(): {e}")
            raise

        local_apps = self.get_local_apps()
        if self.enable_logs:
            print(f"Local apps: {local_apps}")

        if not os.path.exists(self.outdir):
            os.makedirs(self.outdir)
        else:
            overwrite = (
                input(
                    f"Directory {self.outdir} already exists. Do you want to overwrite it? (yes/no): "
                )
                .strip()
                .lower()
            )
            if overwrite not in ["y", "yes"]:
                print(
                    "Operation cancelled. Use the --outdir parameter to specify a different output directory."
                )
                return

        if self.source == "serializers":
            exporter = SerializersExporter(self.outdir, local_apps, self.enable_logs)
        else:
            exporter = ModelsExporter(self.outdir, local_apps, self.enable_logs)

        exporter.export()


def main():
    parser = argparse.ArgumentParser(
        description="Export Django models and serializers to TypeScript interfaces.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-o",
        "--outdir",
        type=str,
        default="./typescript",
        help="Output directory for the TypeScript files.",
    )
    parser.add_argument(
        "-e",
        "--exclude",
        type=str,
        nargs="*",
        default=[],
        help="List of apps to exclude from export.",
    )
    parser.add_argument(
        "-l", "--logs", action="store_true", help="Enable detailed logs."
    )
    parser.add_argument(
        "-s",
        "--source",
        type=str,
        choices=["serializers", "models"],
        default="serializers",
        help="Source to export: serializers or models",
    )
    parser.add_argument(
        "-v", "--version", action="version", version="django-ts-exporter 0.1.0"
    )
    args = parser.parse_args()

    exporter = TypeScriptExporter(args.outdir, args.exclude, args.logs, args.source)
    exporter.run()


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="django-ts-exporter",
    version="0.3.0",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["Django>=3.0", "djangorestframework>=3.11"],
    entry_points={
        "console_scripts": [
            "django-ts-exporter=django_ts_exporter.exporter:main",
        ],
    },
    author="Gabriel Martinusso",
    author_email="gabrielbramos@outlook.com",
    description="Export Django models and serializers to TypeScript interfaces",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/gmartinu/django-ts-exporter",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import os
import inspect
import importlib
from django.apps import apps
from django.db import models
from .utils import create_directory


class ModelsExporter:
    def __init__(self, outdir, apps, enable_logs):
        self.outdir = outdir
        self.apps = apps
        self.enable_logs = enable_logs
        self.any_fields_log = []

    def export(self):
        for app in self.apps:
            app_config = apps.get_app_config(app.split(".")[-1])
            models_module = f"{app_config.name}.models"

            if importlib.util.find_spec(models_module):
                module = importlib.import_module(models_module)
                self.export_models(module, app_config)

            models_path = os.path.join(app_config.path, "models")
            if os.path.isdir(models_path):
                for root, _, files in os.walk(models_path):
                    for file in files:
                        if file.endswith(".py") and file != "__init__.py":
                            module_name = f"{app_config.name}.models.{file[:-3]}"
                            module = importlib.import_module(module_name)
                            self.export_models(module, app_config)

        if self.enable_logs:
            self.log_any_fields()

    def export_models(self, module, app_config):
        app_outdir = os.path.join(self.outdir, app_config.name)
        create_directory(app_outdir)

        for name, obj in inspect.getmembers(module, inspect.isclass):
            if issubclass(obj, models.Model) and obj.__module__ == module.__name__:
                ts_interface, imports = self.convert_model_to_ts_interface(
                    name, obj, app_config.name
                )
                self.write_interface(app_outdir, name, ts_interface, imports)

    def get_related_type_and_import(
        self, related_model, related_serializer, app_name, model_name
    ):
        if related_serializer != model_name:  # Avoid self-import
            related_model_name = (
                related_model
                if isinstance(related_model, str)
                else related_model.__name__
            )
            related_app_label = (
                related_model._meta.app_label
                if hasattr(related_model, "_meta")
                else app_name
            )

            if related_app_label == app_name:
                import_path = f"./{related_model_name}.ts"
            else:
                import_path = f"../{related_app_label}/{related_model_name}.ts"
            return (
                related_model_name,
                f'import {{ {related_model_name} }} from "{import_path}";',
            )
        else:
            return related_serializer, None

    def get_ts_type(self, field, field_name, app_name, model_name):
        if isinstance(field, models.CharField):
            return "string", None
        elif isinstance(field, models.IntegerField):
            return "number", None
        elif isinstance(field, models.BooleanField):
            return "boolean", None
        elif isinstance(field, models.DateTimeField):
            return "string", None  # DateTimeField can be represented as ISO 8601 string
        elif isinstance(field, models.DateField):
            return "string", None  # DateField can be represented as ISO 8601 string
        elif isinstance(field, models.DecimalField):
            return "number", None  # DecimalField can be represented as number
        elif isinstance(field, models.UUIDField):
            return "string", None  # UUIDField can be represented as string
        elif isinstance(field, models.JSONField):
            return (
                "{ [key: string]: any }",
                None,
            )  # JSONField can be represented as dictionary
        elif isinstance(field, models.ForeignKey):
            related_model = field.related_model
            related_serializer = (
                related_model
                if isinstance(related_model, str)
                else related_model.__name__
            )
            return self.get_related_type_and_import(
                related_model, related_serializer, app_name, model_name
            )
        elif isinstance(field, models.ManyToManyField):
            related_model = field.related_model
            related_serializer = (
                related_model
                if isinstance(related_model, str)
                else related_model.__name__
            )
            ts_type, import_statement = self.get_related_type_and_import(
                related_model, related_serializer, app_name, model_name
            )
            return f"{ts_type}[]", import_statement
        elif isinstance(field, models.OneToOneField):
            related_model = field.related_model
            related_serializer = (
                related_model
                if isinstance(related_model, str)
                else related_model.__name__
            )
            return self.get_related_type_and_import(
                related_model, related_serializer, app_name, model_name
            )
        elif isinstance(field, models.FileField):
            return "File", None
        return "any", None

    def handle_field_with_choices(self, field, field_name, model_name):
        enum_name = f"{model_name}{field_name.capitalize()}Enum"
        choices = field.choices
        enum_definition = f"export enum {enum_name} {{\n"
        for value, label in choices.items():
            enum_definition += (
                f"  {label.replace(' ', '_').replace('�', 'í')} = '{value}',\n"
            )
        enum_definition += "}\n"
        return enum_name, enum_definition

    def convert_model_to_ts_interface(self, name, model, app_name):
        fields = model._meta.get_fields()
        ts_fields = []
        imports = set()
        enums = set()

        for field in fields:
            if isinstance(field, models.Field):
                ts_type, enum_definition = self.get_ts_type(
                    field, field.name, app_name, name
                )
                ts_fields.append(f"{field.name}: {ts_type};")

                if ts_type == "any":
                    self.any_fields_log.append(f"{app_name}.{name}.{field.name}")
                if enum_definition:
                    if "export enum" in enum_definition:
                        enums.add(enum_definition)
                    elif enum_definition:
                        imports.add(enum_definition)

        import_statements = "\n".join(sorted(imports))
        enum_statements = "\n".join(sorted(enums))

        interface = (
            f"export interface {name} {{\n  " + "\n  ".join(ts_fields) + "\n}\n\n"
        )

        result = ""

        if import_statements:
            result = f"{import_statements}\n\n"

        if enum_statements:
            result = result + f"{enum_statements}\n\n"

        if interface:
            result = result + f"{interface}"

        return result, list(imports)

    def write_interface(self, app_outdir, name, ts_interface, imports):
        interface_path = os.path.join(app_outdir, f"{name}.ts")
        with open(interface_path, "w", encoding="utf-8") as f:
            f.write(ts_interface)

    def log_any_fields(self):
        if self.any_fields_log:
            print("\nFields with 'any' type detected:")
            for field in self.any_fields_log:
                print(field)
        else:
            print("\nNo fields with 'any' type detected.")



import unittest
from django.test import TestCase
from django.apps import apps
from django.conf import settings
from rest_framework import serializers
from django_ts_exporter.serializers_exporter import SerializersExporter
from tests.models import RelatedModel, TestModel

# Configure settings for the Django test environment if not already configured
if not settings.configured:
    settings.configure(
        INSTALLED_APPS=[
            "django.contrib.contenttypes",
            "django.contrib.auth",
            "tests",
        ],
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": ":memory:",
            }
        },
    )

apps.populate(settings.INSTALLED_APPS)


class SerializersExporterTestCase(TestCase):
    def test_serializer_export(self):
        exporter = SerializersExporter(
            outdir="./tests/typescript/", apps=["tests"], enable_logs=True
        )
        exporter.export()

        with open(
            "./tests/typescript/tests/TestModelSerializer.ts", "r", encoding="utf-8"
        ) as f:
            content = f.read()

        expected_content = """import { RelatedModelSerializer } from "./RelatedModelSerializer.ts";

export interface TestModelSerializer {
  id: number;
  char_field: string;
  integer_field: number;
  boolean_field: boolean;
  datetime_field: string;
  date_field: string;
  decimal_field: number;
  uuid_field: string;
  json_field: { [key: string]: any };
  file_field: File;
  foreign_key: RelatedModelSerializer;
  one_to_one: RelatedModelSerializer;
  many_to_many: RelatedModelSerializer[];
}

"""
        self.assertEqual(content.strip(), expected_content.strip())


if __name__ == "__main__":
    unittest.main()



from django.db import models


class RelatedModel(models.Model):
    name = models.CharField(max_length=100)

    class Meta:
        app_label = "tests"


class TestModel(models.Model):
    char_field = models.CharField(max_length=100)
    integer_field = models.IntegerField()
    boolean_field = models.BooleanField()
    datetime_field = models.DateTimeField()
    date_field = models.DateField()
    decimal_field = models.DecimalField(max_digits=10, decimal_places=2)
    uuid_field = models.UUIDField()
    json_field = models.JSONField()
    foreign_key = models.ForeignKey(
        RelatedModel, on_delete=models.CASCADE, related_name="fk_related"
    )
    many_to_many = models.ManyToManyField(RelatedModel, related_name="m2m_related")
    one_to_one = models.OneToOneField(
        RelatedModel, on_delete=models.CASCADE, related_name="o2o_related"
    )
    file_field = models.FileField()

    class Meta:
        app_label = "tests"
